//
//  ViewController.m
//  SettingsTableView
//
//  Created by Aron Bury on 31/03/13.
//  Copyright (c) 2013 Aron Bury. All rights reserved.
//

#import "ViewController.h"
#import "GroceryItem.h"
#import "GroceryTableViewCell.h"

#define kTableViewCellIdent @"groceryCell"

#define kSectionTitleKey @"grocerySectionTitle"
#define kProductsKey @"groceryProducts"

@interface ViewController () <UITableViewDataSource, UITableViewDelegate, UIAlertViewDelegate>

@property (nonatomic, strong) NSMutableArray *categories;

@property (nonatomic, strong) UIBarButtonItem *editButton;
@property (nonatomic, strong) UIBarButtonItem *doneButton;
@property (nonatomic, strong) UIBarButtonItem *addButton;

- (void)editButtonSelected:(id)sender;
- (void)doneButtonSelected:(id)sender;
- (void)addButtonSelected:(id)sender;

- (GroceryItem *)itemForIndexPath:(NSIndexPath *)indexPath;
- (NSMutableArray *)productArrayForIndexPath:(NSIndexPath *)path;

@end

@implementation ViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.title = @"Groceries";
    self.groceryTableView.delegate = self;
    self.groceryTableView.dataSource = self;
    
    self.editButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemEdit target:self action:@selector(editButtonSelected:)];
    
    self.doneButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(doneButtonSelected:)];
    
    self.addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(addButtonSelected:)];
    
    [self.navigationItem setRightBarButtonItem:self.addButton];
    [self.navigationItem setLeftBarButtonItem:self.editButton];
    
    NSMutableDictionary *basics = @{kSectionTitleKey : @"Basics", kProductsKey : @[].mutableCopy}.mutableCopy;
    NSMutableDictionary *foodSection = @{kSectionTitleKey : @"Food", kProductsKey : @[].mutableCopy}.mutableCopy;
    NSMutableDictionary *bathroomSection = @{kSectionTitleKey : @"Bathroom", kProductsKey : @[].mutableCopy}.mutableCopy;
    
    self.categories = @[basics, foodSection, bathroomSection].mutableCopy;
}

#pragma mark -
#pragma mark - Private Methods
- (void)editButtonSelected:(id)sender
{
    [self.navigationItem setRightBarButtonItem:nil animated:YES];
    [self.groceryTableView setEditing:YES animated:YES];
    [self.navigationItem setLeftBarButtonItem:self.doneButton animated:YES];
}

- (void)doneButtonSelected:(id)sender
{
    [self.navigationItem setRightBarButtonItem:self.addButton animated:YES];
    [self.groceryTableView setEditing:NO animated:YES];
    [self.navigationItem setLeftBarButtonItem:self.editButton animated:YES];
}

- (void)addButtonSelected:(id)sender
{
    UIAlertView *addAlertView = [[UIAlertView alloc] initWithTitle:@"Add Item" message:@"Enter the product name" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"ok", nil];
    addAlertView.alertViewStyle = UIAlertViewStylePlainTextInput;
    
    [addAlertView show];
}

- (GroceryItem *)itemForIndexPath:(NSIndexPath *)indexPath
{    
    return [[self productArrayForIndexPath:indexPath] objectAtIndex:indexPath.row];
}

- (NSMutableArray *)productArrayForIndexPath:(NSIndexPath *)path
{
    NSDictionary *category = [self.categories objectAtIndex:path.section];
    return [category objectForKey:kProductsKey];
}

#pragma mark -
#pragma mark - UITableVieDataSource Methods
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return self.categories.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSDictionary *category = [self.categories objectAtIndex:section];
    NSMutableArray *products = [category objectForKey:kProductsKey];
    
    return products.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    GroceryTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kTableViewCellIdent];
    
    if (!cell)
    {
        cell = [[GroceryTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kTableViewCellIdent];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
    }
    
    GroceryItem *item = [self itemForIndexPath:indexPath];
    cell.textLabel.text = item.itemName;
    
    [cell setPickedUp:item.pickedUp animated:NO];
    
    return cell;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    NSDictionary *dict = [self.categories objectAtIndex:section];
    return [dict objectForKey:kSectionTitleKey];
}

#pragma mark -
#pragma mark - UITableViewDelegate Methods
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath
{
    NSMutableArray *oldproducts = [self productArrayForIndexPath:sourceIndexPath];
    NSMutableArray *newProducts = [self productArrayForIndexPath:destinationIndexPath];
    
    GroceryItem *movedItem = [oldproducts objectAtIndex:sourceIndexPath.row];

    [oldproducts removeObject:movedItem];
    [newProducts insertObject:movedItem atIndex:destinationIndexPath.row];
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSMutableArray *products = [self productArrayForIndexPath:indexPath];
    
    if (editingStyle == UITableViewCellEditingStyleDelete)
    {
        [products removeObjectAtIndex:indexPath.row];        
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    GroceryItem *item = [self itemForIndexPath:indexPath];
    item.pickedUp = !item.pickedUp;
    
    GroceryTableViewCell *cell = (GroceryTableViewCell *)[tableView cellForRowAtIndexPath:indexPath];
    [cell setPickedUp:item.pickedUp animated:YES];
}

#pragma mark -
#pragma mark - UIALertViewDelegate Methods
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex == 0)
        return;
    
    NSString *itemName = [alertView textFieldAtIndex:0].text;
    
    GroceryItem *newItem = [[GroceryItem alloc] init];
    newItem.itemName = itemName;
    
    NSMutableArray *basics = [self productArrayForIndexPath:0];
    [basics insertObject:newItem atIndex:0];
    
    [self.groceryTableView insertRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:0 inSection:0]] withRowAnimation:UITableViewRowAnimationAutomatic];
}
@end
